@@ -12,6 +12,7 @@ from distributed import wait
 import coiled
 from .chess_util import ChessUtil
 from dask.distributed import TimeoutError
+import multiprocessing
 
 class ChessDaskCluster:
     "Class for using dask tasks to parallelize calculation of power of each square"
@@ -56,12 +57,15 @@ class ChessDaskCluster:
                 "max_white_value": gamemax_controlwhite, "max_black_value": gamemax_controlblack}
 
     @staticmethod
-    def run_in_parallel(game, game_no, timeout):
+    def analyse_game_in_worker(game_dict):
         """
         For the given game, for every ply, generate tasks to be run in parallel in a dask cluster.
         One task is created per square to find the control of both the sides. A worker client is
         used here because this method itself is run inside a worker
         """
+        game = game_dict["game"]
+        game_no = game_dict["game_no"]
+        timeout = game_dict["timeout"]
         tasks_for_game = ChessUtil.generate_ply_info_list_for_game(game)
         worker_client = get_client()
         task_futures = worker_client.map(ChessUtil.find_control_for_square, tasks_for_game["ply_info_list"])
@@ -74,6 +78,17 @@ class ChessDaskCluster:
         except TimeoutError:
             print("Game timed out: " + str(game_no))
         return game_data
+    
+    @staticmethod
+    def run_in_parallel_in_worker(game_batch, is_local_cluster):
+        if is_local_cluster == True:
+            results = []
+            for game in game_batch:
+                results.append(ChessDaskCluster.analyse_game_in_worker(game))
+            return results
+        else:
+            pool = multiprocessing.Pool()
+            return pool.map(ChessDaskCluster.analyse_game_in_worker, game_batch)
 
     def __init__(self):
         config_file = os.path.join(os.getcwd(), "config", "config.yaml")
@@ -122,23 +137,28 @@ class ChessDaskCluster:
         game_no = 0
         game_futures = []
         results = []
+        game_master_list = []
+        timeout = self.config_values["timeout_per_game"]
         for game in game_list:
-            game_futures.append(self.client.submit(ChessDaskCluster.run_in_parallel,
-                                                   *(game, game_no), self.config_values["timeout_per_game"]))
+            game_master_list.append({"game": game, "game_no": game_no, "timeout": timeout})
             game_no = game_no + 1
-            if game_no % self.config_values["game_batch_size"] == 0:
+        index = 1
+        for game_batch in self.batch(game_master_list, 4):
+            game_futures.append(self.client.submit(ChessDaskCluster.run_in_parallel_in_worker,
+                                                   game_batch, self.config_values["use_local_cluster"]))
+            if index % self.config_values["game_batch_size"] == 0:
                 for future in game_futures:
-                    try:
-                        result = future.result(self.config_values["timeout_per_game"])
-                        results.append(result)
-                    except TimeoutError:
-                        print("Game timed out")
-                print("Completed till " + str(game_no))
+                    result = future.result()
+                    results.extend(result)
+                print("Completed batches till " + str(index * 4))
+            index = index + 1
         for future in game_futures:
-            try:
-                result = future.result(self.config_values["timeout_per_game"])
-                results.append(result)
-            except TimeoutError:
-                print("Game timed out")
+            result = future.result()
+            results.extend(result)
         print("Completed all")
         return results
+
+    def batch(self, iterable, n=1):
+        l = len(iterable)
+        for ndx in range(0, l, n):
+            yield iterable[ndx:min(ndx + n, l)]
\ No newline at end of file
